/************************************************************************/
/* 		This code is part of 'CutterFF'				*/
/*		a tool for cutting video files with 'FFmpeg' and 'GTK'	*/
/*		Copyright (c) 2017 .. 2019,2021,2023 Harald Foerster	*/
/*		(harald_foerster@users.sourceforge.net)			*/
/*		Distributed under GPL 2 or later			*/
/************************************************************************/

#ifndef CFF_MACROS_GTK__H
#define CFF_MACROS_GTK__H

#include "cff_tools-gtk.h"

/*
#include <gtk/gtk.h>
*/


typedef struct
{
	const char *fname;
	const char *loglevel;
	int        threads;

} cff_init_param_t;

#ifdef HAVE_GMENU
void cff_gui_init(GtkApplication *app, const cff_init_param_t *param);
#else
void cff_gui_init(const cff_init_param_t *param);
#endif


/*
	--------
	 Macros
	--------
*/

#if GDK_MAJOR_VERSION == 2

#define Gui_KEY_BackSpace	GDK_BackSpace
#define Gui_KEY_Escape		GDK_Escape
#define Gui_KEY_Return		GDK_Return
#define Gui_KEY_KP_1		GDK_KP_1
#define Gui_KEY_KP_2		GDK_KP_2
#define Gui_KEY_KP_3		GDK_KP_3
#define Gui_KEY_KP_4		GDK_KP_4
#define Gui_KEY_KP_6		GDK_KP_6
#define Gui_KEY_KP_7		GDK_KP_7
#define Gui_KEY_KP_8		GDK_KP_8
#define Gui_KEY_KP_9		GDK_KP_9
#define Gui_KEY_Home		GDK_Home
#define Gui_KEY_End		GDK_End
#define Gui_KEY_Page_Up		GDK_Page_Up
#define Gui_KEY_Page_Down	GDK_Page_Down
#define Gui_KEY_Up		GDK_Up
#define Gui_KEY_Down		GDK_Down
#define Gui_KEY_Left		GDK_Left
#define Gui_KEY_Right		GDK_Right
#define Gui_KEY_KP_Home		GDK_KP_Home
#define Gui_KEY_KP_End		GDK_KP_End
#define Gui_KEY_KP_Page_Up	GDK_KP_Page_Up
#define Gui_KEY_KP_Page_Down	GDK_KP_Page_Down
#define Gui_KEY_KP_Up		GDK_KP_Up
#define Gui_KEY_KP_Down		GDK_KP_Down
#define Gui_KEY_KP_Left		GDK_KP_Left
#define Gui_KEY_KP_Right	GDK_KP_Right

#else /* GDK_MAJOR_VERSION != 2 */

#define Gui_KEY_BackSpace	GDK_KEY_BackSpace
#define Gui_KEY_Escape		GDK_KEY_Escape
#define Gui_KEY_Return		GDK_KEY_Return
#define Gui_KEY_KP_1		GDK_KEY_KP_1
#define Gui_KEY_KP_2		GDK_KEY_KP_2
#define Gui_KEY_KP_3		GDK_KEY_KP_3
#define Gui_KEY_KP_4		GDK_KEY_KP_4
#define Gui_KEY_KP_6		GDK_KEY_KP_6
#define Gui_KEY_KP_7		GDK_KEY_KP_7
#define Gui_KEY_KP_8		GDK_KEY_KP_8
#define Gui_KEY_KP_9		GDK_KEY_KP_9
#define Gui_KEY_Home		GDK_KEY_Home
#define Gui_KEY_End		GDK_KEY_End
#define Gui_KEY_Page_Up		GDK_KEY_Page_Up
#define Gui_KEY_Page_Down	GDK_KEY_Page_Down
#define Gui_KEY_Up		GDK_KEY_Up
#define Gui_KEY_Down		GDK_KEY_Down
#define Gui_KEY_Left		GDK_KEY_Left
#define Gui_KEY_Right		GDK_KEY_Right
#define Gui_KEY_KP_Home		GDK_KEY_KP_Home
#define Gui_KEY_KP_End		GDK_KEY_KP_End
#define Gui_KEY_KP_Page_Up	GDK_KEY_KP_Page_Up
#define Gui_KEY_KP_Page_Down	GDK_KEY_KP_Page_Down
#define Gui_KEY_KP_Up		GDK_KEY_KP_Up
#define Gui_KEY_KP_Down		GDK_KEY_KP_Down
#define Gui_KEY_KP_Left		GDK_KEY_KP_Left
#define Gui_KEY_KP_Right	GDK_KEY_KP_Right

#endif /* #if GDK_MAJOR_VERSION == 2 .. #else */


#if GTK_MAJOR_VERSION == 2
#define Gui_ALIGN_START		((gfloat) 0.0)
#define Gui_ALIGN_CENTER	((gfloat) 0.5)
#else
#define Gui_ALIGN_START		GTK_ALIGN_START
#define Gui_ALIGN_CENTER	GTK_ALIGN_CENTER
#endif

#ifdef GDK_PRIORITY_REDRAW
#define Gui_PRIORITY_REDRAW	GDK_PRIORITY_REDRAW
#else
#define Gui_PRIORITY_REDRAW	(G_PRIORITY_HIGH_IDLE + 20)
#endif


/*
	*
	* ---------------
	*  Signal Macros
	* ---------------
	*
*/

#define GuiSignalConnect(instance, name, func, data) \
	g_signal_connect((gpointer) (instance), (name), \
					G_CALLBACK(func), (gpointer) (data))
#define GuiSignalConnectSwapped(instance, name, func, swap_obj) \
	g_signal_connect_swapped((gpointer) (instance), (name), \
					G_CALLBACK(func), G_OBJECT(swap_obj))
#define GuiSignalEmitStopByName(instance, name) \
	g_signal_stop_emission_by_name((gpointer) (instance), (name))
#define GuiSignalHandlerBlock(instance, id) \
	g_signal_handler_block((gpointer) (instance), (id))
#define GuiSignalHandlerUnblock(instance, id) \
	g_signal_handler_unblock((gpointer) (instance), (id))
#define GuiSignalHandlerDisconnect(instance, id) \
	g_signal_handler_disconnect((gpointer) (instance), (id))

#ifdef G_SOURCE_CONTINUE
#define Gui_SOURCE_CONTINUE	G_SOURCE_CONTINUE
#else
#define Gui_SOURCE_CONTINUE	TRUE
#endif

#ifdef G_SOURCE_REMOVE
#define Gui_SOURCE_REMOVE	G_SOURCE_REMOVE
#else
#define Gui_SOURCE_REMOVE	FALSE
#endif


/*
	*
	* ---------
	*  Threads
	* ---------
	*
*/

#if GLIB_MAJOR_VERSION == 2 && GLIB_MINOR_VERSION < 32

#define GuiThreadNew(func, data) \
	g_thread_create((GThreadFunc) (func), (data), TRUE, NULL)

#define GuiMutex	 GStaticMutex

#define GuiMutexInit(mutex) \
	g_static_mutex_init(mutex)

#define GuiMutexFree(mutex) \
	g_static_mutex_free(mutex)

#define GuiMutexLock(mutex) \
	g_static_mutex_lock(mutex)

#define GuiMutexTryLock(mutex) \
	g_static_mutex_trylock(mutex)

#define GuiMutexUnlock(mutex) \
	g_static_mutex_unlock(mutex)

#else /* GLIB_MAJOR_VERSION != 2 || GLIB_MINOR_VERSION >= 32 */

#define GuiThreadNew(func, data) \
	g_thread_try_new(NULL, (GThreadFunc) (func), (data), NULL)

#define GuiMutex	 GMutex

#define GuiMutexInit(mutex) \
	g_mutex_init(mutex)

#define GuiMutexFree(mutex) \
	g_mutex_clear(mutex)

#define GuiMutexLock(mutex) \
	g_mutex_lock(mutex)

#define GuiMutexTryLock(mutex) \
	g_mutex_trylock(mutex)

#define GuiMutexUnlock(mutex) \
	g_mutex_unlock(mutex)

#endif /* #if GLIB_MAJOR_VERSION == 2 && GLIB_MINOR_VERSION < 32 .. #else */


/*
	*
	* ------
	*  GLib
	* ------
	*
*/

#ifdef HAVE_GMENU
#define GuiMemDup(ptr, size) \
	g_memdup2((ptr), (size))
#else
#define GuiMemDup(ptr, size) \
	g_memdup((ptr), (size))
#endif


/*
	*
	* ---------------
	*  Miscellaneous
	* ---------------
	*
*/

#if GTK_MAJOR_VERSION == 2
#define GuiImageGetDialogIcon(name) \
	gtk_image_new_from_stock((name), GTK_ICON_SIZE_DIALOG)
#elif GTK_MAJOR_VERSION == 3
#define GuiImageGetDialogIcon(name) \
	gtk_image_new_from_icon_name((name), GTK_ICON_SIZE_DIALOG)
#else
#define GuiImageGetDialogIcon(name) \
	gtk_image_new_from_icon_name(name)
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiMiscSetAlignment(widget, xalign, yalign) \
	gtk_misc_set_alignment(GTK_MISC(widget), (xalign), (yalign))
#else
#define GuiMiscSetAlignment(widget, xalign, yalign) \
	g_object_set(G_OBJECT(widget), "halign", (xalign), \
			"valign", (yalign), NULL)
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiMiscSetPadding(widget, padding) \
	gtk_misc_set_padding(GTK_MISC(widget), (padding), (padding))
#elif GTK_MAJOR_VERSION == 3
#define GuiMiscSetPadding(widget, padding) \
	g_object_set(G_OBJECT(widget), "margin", (padding), NULL)
#else
#define GuiMiscSetPadding(widget, padding) \
	cff_guitl_widget_set_border_width((widget), (padding))
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiProgressbarTextHide(pbar) \
	gtk_progress_bar_set_text((pbar), NULL)
#else
#define GuiProgressbarTextHide(pbar) \
	gtk_progress_bar_set_show_text((pbar), 0)
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiHScaleNew(adjust) \
	gtk_hscale_new(adjust)
#else
#define GuiHScaleNew(adjust) \
	gtk_scale_new(GTK_ORIENTATION_HORIZONTAL, (adjust))
#endif

#if GTK_MAJOR_VERSION == 2 || GTK_MAJOR_VERSION == 3
#define GuiCssLoadFromData(provider, str, error) \
	gtk_css_provider_load_from_data(GTK_CSS_PROVIDER(provider), \
						(str), -1, (error))
#else
#define GuiCssLoadFromData(provider, str, error) \
	gtk_css_provider_load_from_data(GTK_CSS_PROVIDER(provider), \
							(str), -1)
#endif

#if GTK_MAJOR_VERSION == 2 || GTK_MAJOR_VERSION == 3
#define GuiScrolledWindowNew() \
	gtk_scrolled_window_new(NULL, NULL)
#else
#define GuiScrolledWindowNew() \
	gtk_scrolled_window_new()
#endif


/* Adjustment */

#if GTK_MAJOR_VERSION == 2
#define GuiAdjustmentGetUpper(adj) \
	(adj)->upper
#else
#define GuiAdjustmentGetUpper(adj) \
	gtk_adjustment_get_upper(adj)
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiAdjustmentGetValue(adj) \
	(adj)->value
#else
#define GuiAdjustmentGetValue(adj) \
	gtk_adjustment_get_value(adj)
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiAdjustmentGetPageSize(adj) \
	(adj)->page_size
#else
#define GuiAdjustmentGetPageSize(adj) \
	gtk_adjustment_get_page_size(adj)
#endif


/* Button */

#if GTK_MAJOR_VERSION == 2 || GTK_MAJOR_VERSION == 3
#define GuiButtonClicked(button) \
	gtk_button_clicked(GTK_BUTTON(button))
#else
#define GuiButtonClicked(button) \
	gtk_widget_activate(button)
#endif


/* Check-button */

#if GTK_MAJOR_VERSION == 2 || GTK_MAJOR_VERSION == 3
#define GuiCheckButtonSetActive(check_button, active) \
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button), (active))
#else
#define GuiCheckButtonSetActive(check_button, active) \
	gtk_check_button_set_active(GTK_CHECK_BUTTON(check_button), (active))
#endif

#if GTK_MAJOR_VERSION == 2 || GTK_MAJOR_VERSION == 3
#define GuiCheckButtonGetActive(check_button) \
	gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button))
#else
#define GuiCheckButtonGetActive(check_button) \
	gtk_check_button_get_active(GTK_CHECK_BUTTON(check_button))
#endif


/* Color */

#if GTK_MAJOR_VERSION == 2
#define GuiColorParse(spec, color) \
	gdk_color_parse((spec), (color))
#else
#define GuiColorParse(spec, color) \
	gdk_rgba_parse((color), (spec))
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiColorToString(color) \
	gdk_color_to_string(color)
#else
#define GuiColorToString(color) \
	gdk_rgba_to_string(color)
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiColorButtonNewWithColor(color) \
	gtk_color_button_new_with_color(color)
#else
#define GuiColorButtonNewWithColor(color) \
	gtk_color_button_new_with_rgba(color)
#endif


/* Combo */

#if GTK_MAJOR_VERSION == 2
#define GuiOptionMenu	GtkOptionMenu
#else
#define GuiOptionMenu	GtkComboBox
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiOptionMenuTextNew() \
	gtk_option_menu_new()
#else
#define GuiOptionMenuTextNew() \
	gtk_combo_box_text_new()
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiOptionMenuTextAppend(option, menu, index, str) \
{ \
	GtkWidget *mi = gtk_menu_item_new_with_label(str); \
	gtk_menu_append(GTK_MENU(menu), mi); \
	gtk_widget_show(mi); \
}
#else
#define GuiOptionMenuTextAppend(option, menu, index, str) \
	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(option), str)
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiOptionMenuGetActive(option) \
	gtk_option_menu_get_history(GTK_OPTION_MENU(option))
#else
#define GuiOptionMenuGetActive(option) \
	gtk_combo_box_get_active(GTK_COMBO_BOX(option))
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiOptionMenuSetActive(option, index) \
	gtk_option_menu_set_history(GTK_OPTION_MENU(option), (index))
#else
#define GuiOptionMenuSetActive(option, index) \
	gtk_combo_box_set_active(GTK_COMBO_BOX(option), (index))
#endif


/* Container */

#if GTK_MAJOR_VERSION == 2 || GTK_MAJOR_VERSION == 3
#define GuiBoxAddChild(box, child) \
	gtk_container_add(GTK_CONTAINER(box), (child))
#else
#define GuiBoxAddChild(box, child) \
	gtk_box_append(GTK_BOX(box), (child))
#endif

#if GTK_MAJOR_VERSION == 2 || GTK_MAJOR_VERSION == 3
#define GuiFrameAddChild(frame, child) \
	gtk_container_add(GTK_CONTAINER(frame), (child))
#else
#define GuiFrameAddChild(frame, child) \
	gtk_frame_set_child(GTK_FRAME(frame), (child))
#endif

#if GTK_MAJOR_VERSION == 2 || GTK_MAJOR_VERSION == 3
#define GuiScrolledWindowAddChild(scrolled_window, child) \
	gtk_container_add(GTK_CONTAINER(scrolled_window), (child))
#else
#define GuiScrolledWindowAddChild(scrolled_window, child) \
	gtk_scrolled_window_set_child(GTK_SCROLLED_WINDOW(scrolled_window), (child))
#endif

#if GTK_MAJOR_VERSION == 2 || GTK_MAJOR_VERSION == 3
#define GuiWindowAddChild(window, child) \
	gtk_container_add(GTK_CONTAINER(window), (child))
#else
#define GuiWindowAddChild(window, child) \
	gtk_window_set_child(GTK_WINDOW(window), (child))
#endif

#if GTK_MAJOR_VERSION == 2 || GTK_MAJOR_VERSION == 3
#define GuiContainerSetBorderWidth(widget, width) \
	gtk_container_set_border_width(GTK_CONTAINER(widget), (width))
#else
#define GuiContainerSetBorderWidth(widget, width) \
	cff_guitl_widget_set_border_width((widget), (width))
#endif


/* Cursor */

#if GTK_MAJOR_VERSION == 2
#define GuiCursorUnref(cursor) \
	gdk_cursor_unref(cursor)
#else
#define GuiCursorUnref(cursor) \
	g_object_unref(cursor)
#endif

#if GTK_MAJOR_VERSION == 2 || GTK_MAJOR_VERSION == 3
#define GuiWindowSetCursor(window, cursor) \
	gdk_window_set_cursor((window), (cursor))
#else
#define GuiWindowSetCursor(window, cursor) \
	gtk_widget_set_cursor((window), (cursor))
#endif


/* Dialog */

#if GTK_MAJOR_VERSION == 2 || GTK_MAJOR_VERSION == 3
#define GuiDialogRun(dialog) \
	gtk_dialog_run(GTK_DIALOG(dialog))
#else
#define GuiDialogRun(dialog) \
	cff_guitl_dialog_run(dialog)
#endif

#if GTK_MAJOR_VERSION == 2 || GTK_MAJOR_VERSION == 3
#define GuiDialogClose(dialog, response) \
	gtk_widget_destroy(dialog)
#else
#define GuiDialogClose(dialog, response) \
	if ((response) != GTK_RESPONSE_DELETE_EVENT) \
	{ \
		gtk_window_close(GTK_WINDOW(dialog)); \
	}
#endif


/* Entry */

#if GTK_MAJOR_VERSION == 2 || GTK_MAJOR_VERSION == 3
#define GuiEntryGetText(entry) \
	gtk_entry_get_text(entry)
#else
#define GuiEntryGetText(entry) \
	gtk_entry_buffer_get_text(gtk_entry_get_buffer(entry))
#endif

#if GTK_MAJOR_VERSION == 2 || GTK_MAJOR_VERSION == 3
#define GuiEntryGetEditable(entry) \
	GTK_EDITABLE(entry)
#else
#define GuiEntryGetEditable(entry) \
	gtk_editable_get_delegate(GTK_EDITABLE(entry))
#endif

#if GTK_MAJOR_VERSION == 2 || GTK_MAJOR_VERSION == 3
#define GuiEntrySetText(entry, str) \
	gtk_entry_set_text((entry), (str))
#else
#define GuiEntrySetText(entry, str) \
	gtk_entry_buffer_set_text(gtk_entry_get_buffer(entry), (str), -1)
#endif


/* File chooser */

#if GTK_MAJOR_VERSION == 2 || GTK_MAJOR_VERSION == 3
#define GuiFileChooserSetCurrentFolder(chooser, gfile) \
	gtk_file_chooser_set_current_folder_file((chooser), (gfile), NULL)
#else
#define GuiFileChooserSetCurrentFolder(chooser, gfile) \
	gtk_file_chooser_set_current_folder((chooser), (gfile), NULL)
#endif

#if GTK_MAJOR_VERSION == 2 || GTK_MAJOR_VERSION == 3
#define GuiFileChooserGetCurrentFolder(chooser) \
	gtk_file_chooser_get_current_folder_file(chooser)
#else
#define GuiFileChooserGetCurrentFolder(chooser) \
	gtk_file_chooser_get_current_folder(chooser)
#endif

#if GTK_MAJOR_VERSION == 2 || GTK_MAJOR_VERSION == 3
#define GuiFileChooserListFilters(file_chooser) \
	gtk_file_chooser_list_filters(file_chooser)
#else
#define GuiFileChooserListFilters(file_chooser) \
	gtk_file_chooser_get_filters(file_chooser)
#endif


/* Label */

#if GTK_MAJOR_VERSION == 2 || GTK_MAJOR_VERSION == 3
#define GuiLabelSetLineWrap(label, wrap) \
	gtk_label_set_line_wrap(GTK_LABEL(label), (wrap))
#else
#define GuiLabelSetLineWrap(label, wrap) \
	gtk_label_set_wrap(GTK_LABEL(label), (wrap))
#endif


/* Main */

#if GTK_MAJOR_VERSION == 2 || GTK_MAJOR_VERSION == 3
#define GuiMainLevelStart() \
	gtk_main()
#else
#define GuiMainLevelStart() \
	cff_guitl_main_level_start()
#endif

#if GTK_MAJOR_VERSION == 2 || GTK_MAJOR_VERSION == 3
#define GuiMainLevelQuitFunc \
	gtk_main_quit
#else
#define GuiMainLevelQuitFunc \
	cff_guitl_main_level_quit
#endif

#define GuiMainLevelQuit() \
	GuiMainLevelQuitFunc()

#ifdef HAVE_GMENU
#if GTK_MAJOR_VERSION == 3
#define GuiMainLevelGetBusy() \
	(gtk_main_level() != 0)
#else
/* cff_tools-gtk.c */
#define GuiMainLevelGetBusy() \
	(gMainLoop != NULL)
#endif
#else /* ! HAVE_GMENU */
#define GuiMainLevelGetBusy() \
	(gtk_main_level() > 1)
#endif


/* Menu */

#ifdef HAVE_GMENU
#define GuiCheckMenuItemNewWithLabel(str) \
	g_menu_item_new((str), NULL)
#else
#define GuiCheckMenuItemNewWithLabel(str) \
	gtk_check_menu_item_new_with_label(str)
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiCheckMenuItemGetActive(item) \
	(item)->active
#else
#define GuiCheckMenuItemGetActive(item) \
	gtk_check_menu_item_get_active(item)
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiMenuBarDisableShadow(menu) \
{ \
	gtk_rc_parse_string( \
		"style \"cutterff-menubar-style\" {GtkMenuBar::shadow-type = none}" \
		"class \"GtkMenuBar\" style : highest \"cutterff-menubar-style\" "); \
	gtk_widget_set_name((menu), "cutterff-menubar-style"); \
}
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiMenuSeparatorNew() \
	gtk_menu_item_new()
#else
#define GuiMenuSeparatorNew() \
	gtk_separator_menu_item_new()
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiMenuShellGetChildren(menu_shell) \
	GTK_MENU_SHELL(menu_shell)->children
#elif GTK_MAJOR_VERSION == 3
#define GuiMenuShellGetChildren(menu_shell) \
	gtk_container_get_children(GTK_CONTAINER(menu_shell))
#endif

#ifdef HAVE_GMENU
#define GuiMenuShellAppend(menu, item) \
	g_menu_append_item((menu), (item))
#else
#define GuiMenuShellAppend(menu, item) \
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), (item))
#endif

#ifdef HAVE_GMENU
#if GTK_MAJOR_VERSION == 3
#define GuiMenuBarNew(model) \
	gtk_menu_bar_new_from_model(G_MENU_MODEL(model))
#else
#define GuiMenuBarNew(model) \
	gtk_popover_menu_bar_new_from_model(G_MENU_MODEL(model))
#endif
#else
#define GuiMenuBarNew(model) \
	gtk_menu_bar_new()
#endif

#ifdef HAVE_GMENU
#define GuiMenuNew() \
	g_menu_new()
#else
#define GuiMenuNew() \
	gtk_menu_new()
#endif

#ifdef HAVE_GMENU
#define GuiMenuItemNewWithLabel(str) \
	g_menu_item_new((str), NULL)
#else
#define GuiMenuItemNewWithLabel(str) \
	gtk_menu_item_new_with_label(str)
#endif

#ifdef HAVE_GMENU
#define GuiMenuItemSetSubmenu(item, submenu) \
	g_menu_item_set_submenu((item), G_MENU_MODEL(submenu))
#else
#define GuiMenuItemSetSubmenu(item, submenu) \
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), (submenu))
#endif

#ifdef HAVE_GMENU
#define GuiMenuItemSetSensitive(item, sensitive) \
	g_simple_action_set_enabled((item), (sensitive))
#else
#define GuiMenuItemSetSensitive(item, sensitive) \
	gtk_widget_set_sensitive((item), (sensitive))
#endif

#ifdef HAVE_GMENU
#if GTK_MAJOR_VERSION == 3
#define GuiPopupMenuNewFromModel(model) \
	gtk_menu_new_from_model(G_MENU_MODEL(model))
#else
#define GuiPopupMenuNewFromModel(model) \
	gtk_popover_menu_new_from_model_full(G_MENU_MODEL(model), \
						GTK_POPOVER_MENU_NESTED)
#endif
#endif

#ifdef HAVE_GMENU
#if GTK_MAJOR_VERSION == 3
#define GuiPopupMenuShow(popup_menu) \
	gtk_menu_popup_at_pointer(GTK_MENU(popup_menu), NULL);
#else
#define GuiPopupMenuShow(popup_menu) \
	gtk_popover_popup(GTK_POPOVER(popup_menu))
#endif
#endif


/* Widget */

#if GTK_MAJOR_VERSION == 2
#define GuiWidgetGetAllocation(widget, alloc) \
	*(alloc) = (widget)->allocation
#else
#define GuiWidgetGetAllocation(widget, alloc) \
	gtk_widget_get_allocation((widget), (alloc))
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiWidgetGetAllocatedHeight(widget) \
	(widget)->allocation.height
#else
#define GuiWidgetGetAllocatedHeight(widget) \
	gtk_widget_get_allocated_height(widget)
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiWidgetGetAllocatedWidth(widget) \
	(widget)->allocation.width
#else
#define GuiWidgetGetAllocatedWidth(widget) \
	gtk_widget_get_allocated_width(widget)
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiWidgetGetWindow(widget) \
	(widget)->window
#else
#define GuiWidgetGetWindow(widget) \
	gtk_widget_get_window(widget)
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiWidgetSetStateSelected(widget) \
	gtk_widget_set_state((widget), GTK_STATE_PRELIGHT)
#elif GTK_MAJOR_VERSION == 3
#define GuiWidgetSetStateSelected(widget) \
	gtk_widget_set_state_flags((widget), GTK_STATE_FLAG_PRELIGHT, FALSE)
#else
#define GuiWidgetSetStateSelected(widget) \
	gtk_widget_set_state_flags((widget), GTK_STATE_FLAG_SELECTED, FALSE)
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiWidgetUnsetStateSelected(widget) \
	gtk_widget_set_state((widget), GTK_STATE_NORMAL)
#elif GTK_MAJOR_VERSION == 3
#define GuiWidgetUnsetStateSelected(widget) \
	gtk_widget_set_state_flags((widget), GTK_STATE_FLAG_NORMAL, TRUE)
#else
#define GuiWidgetUnsetStateSelected(widget) \
	gtk_widget_unset_state_flags((widget), GTK_STATE_FLAG_SELECTED)
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiWidgetCanDefault(widget) \
	GTK_WIDGET_SET_FLAGS((widget), GTK_CAN_DEFAULT)
#elif GTK_MAJOR_VERSION == 3
#define GuiWidgetCanDefault(widget) \
	gtk_widget_set_can_default((widget), TRUE)
#else
#define GuiWidgetCanDefault(widget) \
	gtk_widget_activate_default(widget)
#endif

#if GTK_MAJOR_VERSION == 2 || GTK_MAJOR_VERSION == 3
#define GuiWidgetSetCanFocus(widget, can_focus) \
	gtk_widget_set_can_focus((widget), (can_focus))
#else
#define GuiWidgetSetCanFocus(widget, can_focus) \
	gtk_widget_set_focusable((widget), (can_focus))
#endif

#if GTK_MAJOR_VERSION == 2 || GTK_MAJOR_VERSION == 3
#define GuiWidgetGrabDefault(window, widget) \
	gtk_widget_grab_default(widget)
#else
#define GuiWidgetGrabDefault(window, widget) \
	gtk_window_set_default_widget(GTK_WINDOW(window), (widget))
#endif

#if GTK_MAJOR_VERSION == 2 || GTK_MAJOR_VERSION == 3
#define GuiWidgetShowAll(widget) \
	gtk_widget_show_all(widget)
#else
#define GuiWidgetShowAll(widget) \
	gtk_widget_show(widget)
#endif


/* Window */

#if GTK_MAJOR_VERSION == 2 || GTK_MAJOR_VERSION == 3
#define GuiWindowNew() \
	gtk_window_new(GTK_WINDOW_TOPLEVEL)
#else
#define GuiWindowNew() \
	gtk_window_new()
#endif

#if GTK_MAJOR_VERSION == 2 || GTK_MAJOR_VERSION == 3
#define GuiWindowCloseFunc \
	gtk_widget_destroy
#else
#define GuiWindowCloseFunc \
	gtk_window_close
#endif

#if GTK_MAJOR_VERSION == 2 || GTK_MAJOR_VERSION == 3
#define GuiWindowClose(widget) \
	gtk_widget_destroy(widget)
#else
#define GuiWindowClose(widget) \
	gtk_window_close(GTK_WINDOW(widget))
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiWindowSetModal(dialog, parent) \
{ \
	gtk_window_set_modal(GTK_WINDOW(dialog), TRUE); \
	gtk_window_set_transient_for(GTK_WINDOW(dialog), \
					GTK_WINDOW(parent)); \
}
#else
#define GuiWindowSetModal(dialog, parent) \
	g_object_set(G_OBJECT(dialog), "modal", TRUE, \
			"transient-for", (parent), NULL)
#endif


/* Boxes */

#if GTK_MAJOR_VERSION == 2
#define GuiHBoxNew(homogeneous, spacing) \
	gtk_hbox_new((homogeneous), (spacing))
#else
#define GuiHBoxNew(homogeneous, spacing) \
	cff_guitl_box_new(GTK_ORIENTATION_HORIZONTAL, \
					(homogeneous), (spacing))
#endif

#if GTK_MAJOR_VERSION == 2 || GTK_MAJOR_VERSION == 3
#define GuiHBoxPackStart(box, child, expand, fill, padding) \
	gtk_box_pack_start(GTK_BOX(box), (child), (expand), (fill), (padding))
#else
#define GuiHBoxPackStart(box, child, expand, fill, padding) \
	g_object_set(G_OBJECT(child), "hexpand", (expand), \
		"halign", (((fill) == FALSE) ? \
			GTK_ALIGN_CENTER : GTK_ALIGN_FILL), \
				"margin-start", (padding), \
					"margin-end", (padding), NULL); \
	gtk_box_append(GTK_BOX(box), (child));
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiVBoxNew(homogeneous, spacing) \
	gtk_vbox_new((homogeneous), (spacing))
#else
#define GuiVBoxNew(homogeneous, spacing) \
	cff_guitl_box_new(GTK_ORIENTATION_VERTICAL, \
					(homogeneous), (spacing))
#endif

#if GTK_MAJOR_VERSION == 2 || GTK_MAJOR_VERSION == 3
#define GuiVBoxPackStart(box, child, expand, fill, padding) \
	gtk_box_pack_start(GTK_BOX(box), (child), (expand), (fill), (padding))
#else
#define GuiVBoxPackStart(box, child, expand, fill, padding) \
	g_object_set(G_OBJECT(child), "vexpand", (expand), \
		"valign", (((fill) == FALSE) ? \
			GTK_ALIGN_CENTER : GTK_ALIGN_FILL), \
				"margin-top", (padding), \
					"margin-bottom", (padding), NULL), \
	gtk_box_append(GTK_BOX(box), (child))
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiHButtonBoxNew() \
	gtk_hbutton_box_new()
#elif GTK_MAJOR_VERSION == 3
#define GuiHButtonBoxNew() \
	gtk_button_box_new(GTK_ORIENTATION_HORIZONTAL)
#else /* GTK_MAJOR_VERSION == 4 */
#define GuiHButtonBoxNew() \
	GuiHBoxNew(TRUE, 0)
#endif

#endif /* CFF_MACROS_GTK__H */
